﻿using System;

namespace David.Academy.ProgrammingInCSharp.CSharpBasics
{
	enum DayOfWeek : long
	{
		/// <summary>
		/// стойността е изрично зададена като 1 (иначе е 0)
		/// </summary>
		Mon = 1,
		/// <summary>
		/// стойността автоматично става 2 (следващото цяло число)
		/// </summary>
		Tue,
		Wed,
		Thu,
		Fri,
		Sat,
		Sun,
	}

	/// <summary>
	/// Изброим списък за състоянието на една новина
	/// </summary>
	/// <remarks>
	/// [Flags] е атрибут, който е силно препоръчителен за изброими списъци, които се използват като битови маски.
	/// Независимите стойности на битовите полета обикновено вървят по степени на двойката.
	/// </remarks>
	/// <example>
	/// Примери за използване на битови полета:
	/// * проверка на флаг/бит (проверяваме дали новината е чернова)
	///   if ((state & NewsState.IsDeleted) == NewsState.IsDeleted) { ... }
	/// * изчистване на флаг/бит (маркираме новина като "белова")
	///   state = state & ~NewsState.IsDraft;
	/// * вдигане на флаг/бит (маркираме новина като одобрена)
	///   state = state | NewsState.IsApproved;
	/// * превключване на флаг/бит (превключваме между публична и непублична новина)
	///   state = state ^ NewsState.IsPublic;
	/// </example>
	[Flags]
	enum NewsState
	{
		/// <summary>
		/// Новината е изтрита или не?
		/// </summary>
		/// <remarks>
		/// Стойността е "00000001" (двоично)
		/// </remarks>
		IsDeleted = 4,
		/// <summary>
		/// Новината е още чернова или не?
		/// </summary>
		/// <remarks>
		/// Стойността е "00000010" (двоично)
		/// </remarks>
		IsDraft = 1,
		/// <summary>
		/// Новината е одобрена или не?
		/// </summary>
		/// <remarks>
		/// Стойността е "00000100" (двоично)
		/// </remarks>
		IsApproved = 2,
		/// <summary>
		/// Новината е публична или не?
		/// </summary>
		/// <remarks>
		/// Стойността е "00001000" (двоично)
		/// </remarks>
		IsPublic = 8,
	}
}
