﻿using System;
using System.IO;
using System.Threading;

namespace David.Academy.ProgrammingInCSharp.CommonMethods
{
	class Program
	{
		static void Main(string[] args)
		{
			string courseName;

			#region string.IsNullOrEmpty & string.IsNullOrWhiteSpace

			while (true)
			{
				Console.Write("Въведете име на курс: ");
				courseName = Console.ReadLine();
				// проверяваме дали низът е null или празен
				//if (string.IsNullOrEmpty(courseName))
				// проверяваме дали низът е null, празен или съставен само от интервали
				if (string.IsNullOrWhiteSpace(courseName))
				{
					Console.WriteLine("Въведен е празен низ.");
				}
				else
				{
					break;
				}
			}

			#endregion

			#region Подкастряне на низове

			courseName = courseName.Trim(' ', '\t', '\n', '$', '%');

			#endregion

			#region Подравняване на низ

			string paddedCourseName = courseName.PadLeft(50, '_');

			#endregion

			#region Форматиране на низ

			string message = string.Format("Име на курса: {0}", paddedCourseName);
			Console.WriteLine(message);

			#endregion

			#region Дължина на низ

			int length = message.Length;
			Console.WriteLine("Дължина на съобщението: {0}", length);

			#endregion

			#region Търсене на символ в низ

			int index = message.IndexOf(':');

			#endregion

			#region Извличане на подниз

			string title = message.Substring(0, index);
			Console.WriteLine(title);

			#endregion

			#region Замяна на подНИЗ в НИЗ с друг НИЗ

			string replacedCourseName = courseName.Replace("Java", "C#");
			Console.WriteLine(replacedCourseName);

			#endregion

			#region Проверка за начало и край на низ

			if (courseName.StartsWith("Programming"))
				Console.WriteLine("Yuppie, you're going to become a programmer, aren't you?");
			else
				Console.WriteLine("Shame on you!");

			if (courseName.EndsWith("C#"))
				Console.WriteLine("Yuppie! C#, D#... anyway...");
			else
				Console.WriteLine("Damn you!");

			#endregion

			#region Съединяване на масив от низове в низ

			string[] lines = new[]
			{
				"Дядо Коледа",
				"Купи ми камионче!",
				"Бях добро дете!",
				"Иванчо",
			};

			string letter = string.Join("\r\n", lines);
			Console.WriteLine(letter);

			#endregion

			#region Разделяне на низ в масив от низове

			string fileContent = @"
using System;

namespace HelloWorld
{
   class Program
   {
      static void Main(string[] args)
      {
          // едноредов коментар
          Console.WriteLine(""Hello, world!"");
      }
   }
}
";
			string[] fileLines = fileContent.Split(new[] { '\r', '\n' });
			for (int i = 0; i < fileLines.Length; i++)
			{
				Console.WriteLine(fileLines[i]);
				if (fileLines[i].TrimStart().StartsWith("//"))
					Console.WriteLine("На ред {0} има едноредов коментар.", i + 1);
			}

			#endregion

			#region Превръщане между малки и големи букви

			Console.WriteLine("Нагоооооре: {0}", courseName.ToUpper());
			Console.WriteLine("Надоооооле: {0}", courseName.ToLower());

			#endregion

			#region Превръщане в низ

			int n = 5;
			string s = n.ToString();

			#endregion

			#region Превръщане на низ

			double a;
			bool result;
			do
			{
				Console.Write("Въведете страна на квадрат a = ");
				s = Console.ReadLine();
				result = double.TryParse(s, out a);
			}
			while (!result);

			Console.WriteLine("a = {0}", a);

			#endregion

			#region Превръщане между типове данни

			int x = 5;
			double y = Convert.ToDouble(x);

			#endregion

			#region Работа с файлове

			File.WriteAllText("letter.txt", letter);
			letter = File.ReadAllText("letter.txt");

			File.WriteAllLines("Program.cs", fileLines);
			fileLines = File.ReadAllLines("Program.cs");

			byte[] fileBytes = File.ReadAllBytes("CommonMethods.exe");
			File.WriteAllBytes("CommonMethods.exe.bak", fileBytes);

			#endregion

			#region Работа с масиви

			byte[] newFileBytes = (byte[])fileBytes.Clone();
			Array.Copy(fileBytes, 0, newFileBytes, 0, 5);
			Array.Sort(fileBytes);
			Array.Clear(fileBytes, 0, 5);

			int indexOfIvancho = Array.IndexOf(lines, "Иванчо");
			// Array.LastIndexOf(lines, "Иванчо");
			// Array.BinarySearch(lines, "Иванчо");

			int[,] matrix = new[,] { { 0, 0 }, { 0, 0 } };
			int maxX = matrix.GetLength(0);
			int maxY = matrix.GetLength(1);

			Array.Reverse(lines);

			#endregion

			#region Работа с конзола

			// изчистване на конзолата
			Console.Clear();
			// издаване на звук
			Console.Beep();
			// големина на прозореца
			Console.SetWindowSize(50, 20);
			// позиция на прозореца
			Console.SetWindowPosition(0, 0);
			// проверка за натиснат клавиш
			if (Console.KeyAvailable)
				Console.WriteLine("Yuppie, key is available!");
			// позиция на курсора
			Console.SetCursorPosition(5, 5);
			// скриване на курсора
			Console.CursorVisible = false;
			// цвят на шрифта и на фона
			Console.ForegroundColor = ConsoleColor.DarkGray;
			Console.BackgroundColor = ConsoleColor.DarkBlue;
			// големина на прозореца
			int height = Console.WindowHeight;
			int width = Console.WindowWidth;
			// заглавие на прозореца
			Console.Title = "Май програм";

			#endregion

			#region Генериране на произволно число и изчакване

			Random r = new Random();
			int randomInt = r.Next(100, 1000);
			double randomDouble = r.NextDouble();
			r.NextBytes(fileBytes);

			// в 30% от случаите печатай нещо
			while (!Console.KeyAvailable)
			{
				int p = r.Next(100);
				// 0-30
				if (p < 30)
					Console.WriteLine("Опаа!");
				// 31-50
				else if (p < 50)
					Console.WriteLine("Опааа #2");
				// 51-60
				else if (p < 60)
					Console.WriteLine("Опааа #3");
				// 61-99
				else
					Console.WriteLine("Опааа #4");

				Thread.Sleep(100);
			}

			#endregion
		}
	}
}
