﻿using System;

namespace David.Academy.ProgrammingInCSharp.Encapsulation
{
	/// <summary>
	/// Клас за точка в равнината
	/// </summary>
	class Point
	{
		// Вътрешно представяне на данните на точката - може да се съхраняват
		// декартови или полярни координати
		private double _x;
		private double _y;
		//private double _angle;
		//private double _radius;

		/// <summary>
		/// Конструктор на точката с декартови координати
		/// </summary>
		/// <param name="x">Абсциса на точката</param>
		/// <param name="y">Ордината на точката</param>
		public Point(double x, double y)
			: this(x, y, true)
		{
		}

		/// <summary>
		/// Конструктор на точката с декартови или полярни координати
		/// </summary>
		/// <param name="value1">Първа координата на точката (абсциса или полярен ъгъл)</param>
		/// <param name="value2">Втора координата на точката (ордината или разстояние от центъра)</param>
		/// <param name="isCartesian">Указва дали подадените координати са декартови или полярни</param>
		public Point(double value1, double value2, bool isCartesian)
		{
			if (isCartesian)
			{
				_x = value1;
				_y = value2;
				//if (value1 == 0)
				//	_angle = 0;
				//else
				//	_angle = Math.Atan(value2 / value1);
				//_radius = Math.Sqrt(Math.Pow(value1, 2) + Math.Pow(value2, 2));
			}
			else
			{
				_x = value2 * Math.Cos(value1);
				_y = value2 * Math.Sin(value1);
				//_angle = value1;
				//_radius = value2;
			}
		}

		/// <summary>
		/// Връща абсцисата на точката в декартова координатна система
		/// </summary>
		/// <returns>Абсциса на точката</returns>
		public double GetX()
		{
			return _x;
			//return _radius * Math.Cos(_angle);
		}

		/// <summary>
		/// Връща ординатата на точката в декартова координатна система
		/// </summary>
		/// <returns>Ордината на точката</returns>
		public double GetY()
		{
			return _y;
			//return _radius * Math.Sin(_angle);
		}

		/// <summary>
		/// Връща полярния ъгъл на точката в полярна координатна система
		/// </summary>
		/// <returns>Полярен ъгъл на точката</returns>
		public double GetAngle()
		{
			if (_x == 0)
				return 0;
			else
				return Math.Atan(_y / _x);
			//return _angle;
		}

		/// <summary>
		/// Връща разстоянието на точката от центъра на полярната координатна система
		/// </summary>
		/// <returns>Разстояние на точката от центъра</returns>
		public double GetRadius()
		{
			return Math.Sqrt(Math.Pow(_x, 2) + Math.Pow(_y, 2));
			//return _radius;
		}
	}
}
