﻿using System;

namespace David.Academy.ProgrammingInCSharp.Encapsulation
{
	/// <summary>
	/// Клас за квадрат. Наследява класа Shape2D
	/// </summary>
	class Square : Shape2D
	{
		// Частно поле за дължина на страната на квадрата
		private double _side;

		/// <summary>
		/// Конструктор на квадрата с параметър: дължина на страната му;
		/// изпълнява защитения конструктор на родителския клас със съответната стойност
		/// за лице на квадрата
		/// </summary>
		/// <param name="side">Дължина на страната на квадрата</param>
		public Square(double side)
			: base(Math.Pow(side, 2))
		{
			// Предизвикване на грешка при изпълнение, ако е подадена неположителна дължина на страна
			if (side <= 0)
				throw new ArgumentException();

			_side = side;
		}

		/// <summary>
		/// Връща дължината на страната на квадрата
		/// </summary>
		/// <returns>Дължина на страната на квадрата</returns>
		public double GetSide()
		{
			return _side;
		}

		/// <summary>
		/// Променя дължината на страната на квадрата
		/// </summary>
		/// <param name="side">Нова дължина на страната на квадрата</param>
		public void SetSide(double side)
		{
			// Предизвикване на грешка при изпълнение, ако е подадена неположителна дължина на страна
			if (side <= 0)
				throw new ArgumentException();

			_side = side;
			// Преизчисляване на лицето на квадрата
			SetArea(Math.Pow(side, 2));
		}
	}
}
