﻿using System;

namespace David.Academy.ProgrammingInCSharp.Inheritance
{
	/// <summary>
	/// Клас за произволна сграда. Базов клас за йерархията
	/// </summary>
	class Building
	{
		// Характеристики, общи за всички сгради: височина, заемана площ и брой етажи
		public double Height;
		public double Area;
		public int FloorCount;

		/// <summary>
		/// Конструктор без параметри
		/// </summary>
		public Building()
		{
			Height = 10;
			Area = 1000;
		}

		/// <summary>
		/// Конструктор с параметри: височина, заемана площ и брой етажи
		/// </summary>
		/// <param name="height">Височина на сградата</param>
		/// <param name="area">Заемана площ</param>
		/// <param name="floorCount">Брой етажи</param>
		public Building(double height, double area, int floorCount)
		{
			Height = height;
			Area = area;
			FloorCount = floorCount;
		}

		/// <summary>
		/// Конструктор с парамтери: височина и заемана площ; броят етажи се изчислява на
		/// базата на височината на сградата
		/// </summary>
		/// <param name="height">Височина на сградата</param>
		/// <param name="area">Заемана площ</param>
		public Building(double height, double area)
			: this(height, area, (int)(height / 3))
		{
		}

		/// <summary>
		/// Разрушава сградата
		/// </summary>
		public void Demolish()
		{
			Console.WriteLine("The building was demolished.");
		}

		/// <summary>
		/// Връща текстово описание на характеристиките на сградата
		/// </summary>
		/// <returns>Текстово описание на характеристиките на сградата</returns>
		public string GetDescription()
		{
			return string.Format("Height: {0}m; Area: {1}m^2; Floors : {2}", Height, Area, FloorCount);
		}
	}
}
