﻿using System;

namespace David.Academy.ProgrammingInCSharp.Inheritance
{
	/// <summary>
	/// Клас за планета. Наследява класа CelestialBody
	/// </summary>
	class Planet : CelestialBody
	{
		// Характеристики, специфични за планетите: дали планетата е населена,
		// вид на планетата, дали има пръстени и брой сателити
		public bool IsInhabited;
		public string Type;
		public bool HasRings;
		public int SatelliteCount;

		/// <summary>
		/// Конструктор с параметри: наименование и брой сателити; изпълнява конструктора
		/// на родителския клас с параметри наименование, маса и разстояние от Земята,
		/// като за масата и разстоянието подава нулеви стойности
		/// </summary>
		/// <param name="name">Наименование на планетата</param>
		/// <param name="satelliteCount">Брой сателити</param>
		public Planet(string name, int satelliteCount)
			: base(name, 0, 0)
		{
			Console.WriteLine("Planet::.ctor(name, satelliteCount)");
			SatelliteCount = satelliteCount;
		}

		/// <summary>
		/// Кара планетата да експлодира
		/// </summary>
		public void Explode()
		{
			Console.WriteLine("BOOM!");
		}

		/// <summary>
		/// Сблъсква планетата с друго небесно тяло, като в резултат от сблъсъка
		/// планетата експлодира
		/// </summary>
		/// <param name="otherBody">Небесното тяло, с което се сблъсква планетата</param>
		public void CollideWith(CelestialBody otherBody)
		{
			Console.WriteLine("{0} collided with {1}.", Name, otherBody.Name);
			Explode();
		}

		/// <summary>
		/// Връща текстово описание на характеристиките на планетата; използва едноименния
		/// метод на родителския клас, за да получи описание на общите за всички небесни тела
		/// характеристики
		/// </summary>
		/// <returns>Текстово описание на характеристиките на планетата</returns>
		public string GetDescription()
		{
			return string.Format("{0}, IsInhabited: {1}, Type: {2}, HasRings: {3}, Satellites: {4}",
				base.GetDescription(),
				IsInhabited,
				Type,
				HasRings,
				SatelliteCount);
		}
	}
}
