﻿namespace David.Academy.ProgrammingInCSharp.Inheritance
{
	/// <summary>
	/// Клас за жилищна сграда. Наследява класа Building
	/// </summary>
	class ResidentialBuilding : Building
	{
		// Характеристики, специфични за жилищните сгради: брой входове и брой жители
		public int EntranceCount;
		public int ResidentCount;

		/// <summary>
		/// Конструкор без параметри; изпълнява конструктора без параметри на родителския клас
		/// </summary>
		public ResidentialBuilding()
		{
		}

		/// <summary>
		/// Конструктор с параметри: височина, заемана площ, брой входове и брой жители;
		/// изпълнява конструктора на родителския клас с параметри височина на и заемана площ
		/// </summary>
		/// <param name="height">Височина на сградата</param>
		/// <param name="area">Заемана площ</param>
		/// <param name="entranceCount">Брой входове</param>
		/// <param name="residentCount">Бровй жители</param>
		public ResidentialBuilding(double height, double area, int entranceCount, int residentCount)
			: base(height, area)
		{
			EntranceCount = entranceCount;
			ResidentCount = residentCount;
		}

		/// <summary>
		/// Регистрира нанасяне на нови жители в сградата
		/// </summary>
		/// <param name="residentCount">Брой на нанеслите се в сградата жители</param>
		public void MovePeopleIn(int residentCount)
		{
			ResidentCount += residentCount;
		}

		/// <summary>
		/// Регистрира изнасяне на жители от сградата
		/// </summary>
		/// <param name="residentCount">Брой на изнеслите се от сградата жители</param>
		public void MovePeopleOut(int residentCount)
		{
			ResidentCount -= residentCount;
		}

		/// <summary>
		/// Връща текстово описание на характеристиките на жилищната сграда; използва
		/// едноименния метод на родителския клас, за да полущи описание на общите
		/// за всички сгради характеристики
		/// </summary>
		/// <returns>Текстово описание на характеристиките на жилищната сграда</returns>
		public string GetDescription()
		{
			return string.Format("{0}; Entrances: {1}; Residents: {2}", base.GetDescription(), EntranceCount, ResidentCount);
		}
	}
}
