﻿using System.Collections;

namespace David.Academy.ProgrammingInCSharp.Enumerations
{
	/// <summary>
	/// Клас за колекция от книги. Имплементира интерфейса IEnumerable
	/// </summary>
	class BookCollection : IEnumerable
	{
		// Частен масив от книги
		private readonly Book[] _books;
		// Брой книги в колекцията
		private int _count;

		/// <summary>
		/// Вътрешен клас за изброяване на колекцията от книги. Имплементира интерфейса IEnumerator
		/// </summary>
		private class Enumerator : IEnumerator
		{
			// Частна референция към колекцията от книги, която се изброява
			private readonly BookCollection _collection;
			// Индекс на елемента, върху който се намира изброяващият обект в момента
			private int _index;

			/// <summary>
			/// Конструктор на обекта за изброяване на колекцията от книги
			/// </summary>
			/// <param name="collection">Колекция от книги, която ще се изброява</param>
			public Enumerator(BookCollection collection)
			{
				_collection = collection;
				// След създаването си, обектът за изброяване на колекция трябва да се намира
				// непосредтвено преди първия елемент в колекцията
				_index = -1;
			}

			/// <summary>
			/// Връща елемента от колекцията от книги, върху който се намира изброяващият обект в момента
			/// </summary>
			public object Current
			{
				get { return _collection._books[_index]; }
			}

			/// <summary>
			/// Придвижва обекта за изброяване на колекцията от книги върху следващия елемент
			/// или след края на колекцията, в случай че няма такъв
			/// </summary>
			/// <returns>Истина, ако обектът за изброяване на колекцията е бил преместен успешно
			/// върху следващия елемент; Неистина, ако няма повече елементи в колекцията</returns>
			public bool MoveNext()
			{
				_index++;
				return (_index <= _collection._count - 1);
			}

			/// <summary>
			/// Връща обекта за изброяване на колекцията от книги в начално състояние
			/// (т.е. непосредствено преди първия елемент в колекцията)
			/// </summary>
			public void Reset()
			{
				_index = -1;
			}
		}

		/// <summary>
		/// Конструктор на колекцията от книги с параметър капацитет на колекцията
		/// </summary>
		/// <param name="capacity">Капацитет на колекцията</param>
		public BookCollection(int capacity)
		{
			_books = new Book[capacity];
		}

		/// <summary>
		/// Добавя книга в колекцията
		/// </summary>
		/// <param name="book">Книга, която трябва да бъде добавена в колекцията</param>
		public void AddBook(Book book)
		{
			_books[_count++] = book;
		}

		/// <summary>
		/// Връща обект за изброяване на колекцията от книги
		/// </summary>
		/// <returns>Обект за изброяване на колекцията от книги, който имплементира интерфейса
		/// IEnumerator</returns>
		public IEnumerator GetEnumerator()
		{
			return new Enumerator(this);
		}
	}
}
