﻿using System;
using System.IO;

namespace David.Academy.ProgrammingInCSharp.Enumerations
{
	/// <summary>
	/// Главен клас на програмата
	/// </summary>
	class Program
	{
		/// <summary>
		/// Главна програма
		/// </summary>
		/// <param name="args">Аргументи от командния ред</param>
		static void Main(string[] args)
		{
			// Създаване на екземпляр на класа BookCollection
			BookCollection bookCollection = new BookCollection(10);
			// Попълване на колекцията от книги с няколко книги
			bookCollection.AddBook(new Book
			{
				Title = "Hobbit",
				Author = "Tolkien",
			});
			bookCollection.AddBook(new Book
			{
				Title = "The Lord of the Rings",
				Author = "Tolkien",
			});

			// Изброяване на колекцията от книги с помощта на конструкцията за цикъл foreach;
			// Това е възможно, тъй като класът BookCollection имплементира интерфейса IEnumerable
			foreach (Book book in bookCollection)
			{
				Console.WriteLine("{0} by {1}", book.Title, book.Author);
			}

			// Използване на using блок за автоматично затваряне на отворения за писане файл Books.txt;
			// Това е възможно, тъй като класът StreamWriter имплементира интерфейса IDisposable
			using (StreamWriter writer = new StreamWriter(@"Books.txt"))
			{
				// Изпечатване на информация за книгите в колекцията във файла
				foreach (Book book in bookCollection)
				{
					writer.WriteLine("{0} by {1}", book.Title, book.Author);
				}
				// След напускане на using блока, програта автоматично затваря и освобождава файла
			}

			// Създаване на екземпляр на класа SongCollection
			SongCollection songCollection = new SongCollection(20);
			// Попълване на колекцията от песни с няколко песни
			songCollection.AddSong(new Song("Lightning Bolt", "Pearl Jam"));
			songCollection.AddSong(new Song("Sirens", "Pearl Jam"));
			songCollection.AddSong(new Song("Обичам те", "Азис"));

			// Използване на using блок за автоматично затваряне на отворения за писане файл Songs.txt;
			// Това е възможно, тъй като класът StreamWriter имплементира интерфейса IDisposable
			using (StreamWriter writer = new StreamWriter("Songs.txt"))
			{
				// Изброяване на колекцията от песни с помощта на конструкцията за цикъл foreach;
				// Това е възможно, тъй като класът SongCollection имплементира интерфейса IEnumerable
				foreach (Song song in songCollection)
				{
					// Изпечатване на характеристиките на песента на конзолата
					song.PrintToConsole();
					// Изпечатване на характеристиките на песента във файла
					song.PrintToWriter(writer);
				}
				// След напускане на using блока, програта автоматично затваря и освобождава файла
			}

			// Използване на using блок за демонстрация на автоматичното изпълняване на
			// метода IDisposable.Dispose() след напускането му
			using (DisposableObject d = new DisposableObject())
			{
				// Произволни операции...
			}

			Console.ReadLine();
		}
	}
}
