﻿using System;
using System.Collections.Generic;

namespace David.Academy.ProgrammingInCSharp.Generics
{
	/// <summary>
	/// Статичен клас с помощни методи за работа със строготипизирани двоични дървета
	/// </summary>
	static class BinaryTreeHelper
	{
		/// <summary>
		/// Шаблонен метод за преобразуване на строготипизирано наредено двоично дърво
		/// към строготипизиран списък, като се запазва наредбата на елементите
		/// </summary>
		/// <typeparam name="T">Тип на данните в нареденото двоично дърво, който трябва
		/// да имплементира интерфейса IComparable</typeparam>
		/// <param name="rootNode">Коренов възел на строготипизираното наредено двоично дърво</param>
		/// <returns>Списък от елементите, съдържащи се в нареденото двоично дърво</returns>
		public static List<T> ConvertTreeToList<T>(BinaryTreeNode<T> rootNode)
			where T : IComparable
		{
			// Създаване на празен списък от елементи от същия тип
			List<T> list = new List<T>();
			// Извикване на помощния метод AddSubtreeToList<T>(BinaryTreeNode<T>, List<T>)
			// върху цялото наредено двоично дърво;
			// При извикването не се указва явно действителния тип на типовия аргумент T,
			// тъй като компилаторът може да го разпознае по типовете на аргументите, подадени на метода
			AddSubtreeToList(rootNode, list);

			return list;
		}

		/// <summary>
		/// Частен шаблонен метод за рекурсивно добавяне на възлите на наредено двоично
		/// поддърво към съществуващ списък от елементи, като се запазва наредбата на елементите
		/// </summary>
		/// <typeparam name="T">Тип на данните в нареденото двоично дърво, който трябва
		/// да имплементира интерфейса IComparable</typeparam>
		/// <param name="node">Възел, от който започва поддървото</param>
		/// <param name="list">Списък, в който трябва да бъдат добавени елементите от възлите
		/// на поддървото</param>
		private static void AddSubtreeToList<T>(BinaryTreeNode<T> node, List<T> list)
			where T : IComparable
		{
			if (node.Left != null)
			{
				// Ако възелът има ляво поддърво, се извиква рекурсивно методът
				// AddSubtreeToList<T>(BinaryTreeNode<T>, List<T>) върху лявото му поддърво;
				// При извикването не се указва явно действителния тип на типовия аргумент T,
				// тъй като компилаторът може да го разпознае по типовете на аргументите, подадени на метода
				AddSubtreeToList(node.Left, list);
			}
			// Добавяне на данните на възела в списъка
			list.Add(node.Data);
			if (node.Right != null)
			{
				// Ако възелът има дясно поддърво, се извиква рекурсивно методът
				// AddSubtreeToList<T>(BinaryTreeNode<T>, List<T>) върху дясното му поддърво;
				// При извикването не се указва явно действителния тип на типовия аргумент T,
				// тъй като компилаторът може да го разпознае по типовете на аргументите, подадени на метода
				AddSubtreeToList(node.Right, list);
			}
		}
	}
}
