﻿using System.Collections.Generic;

namespace David.Academy.ProgrammingInCSharp.Exceptions
{
	class BookDatabaseV2
	{
		private readonly List<BookEntry> _entries;

		public BookDatabaseStatus Status { get; private set; }
		public string StatusText { get; private set; }

		public BookDatabaseV2()
		{
			_entries = new List<BookEntry>();
		}

		public List<BookEntry> GetEntries()
		{
			List<BookEntry> entries = new List<BookEntry>();
			foreach (BookEntry entry in _entries)
			{
				entries.Add(entry);
			}

			Status = BookDatabaseStatus.Ok;
			StatusText = null;

			return entries;
		}

		public BookEntry GetEntry(int index)
		{
			if (index < 0 || index > _entries.Count - 1)
			{
				Status = BookDatabaseStatus.ArgumentError;
				StatusText = "Index out of range";
				return null;
			}

			Status = BookDatabaseStatus.Ok;
			StatusText = null;

			return _entries[index];
		}

		public void AddEntry(BookEntry entry)
		{
			Status = BookDatabaseStatus.Ok;
			StatusText = null;

			_entries.Add(entry);
		}

		public void RemoveEntry(int index)
		{
			if (index < 0 || index > _entries.Count - 1)
			{
				Status = BookDatabaseStatus.ArgumentError;
				StatusText = "Index out of range";
				return;
			}

			Status = BookDatabaseStatus.Ok;
			StatusText = null;

			_entries.RemoveAt(index);
		}

		public List<BookEntry> FindEntries(string name)
		{
			List<BookEntry> entries = new List<BookEntry>();
			foreach (BookEntry entry in _entries)
			{
				if (entry.Name.Contains(name))
					entries.Add(entry);
			}

			Status = BookDatabaseStatus.Ok;
			StatusText = null;

			return entries;
		}
	}
}
