﻿using System;
using System.Collections.Generic;
using System.Reflection;

namespace David.Academy.ProgrammingInCSharp.Reflections
{
	class Program
	{
		static void Main(string[] args)
		{
			//Assembly assembly = Assembly.GetEntryAssembly();
			Assembly assembly = Assembly.LoadFrom(@"C:\\Projects\\Lecture13_Group2\\Exceptions\\bin\\Debug\\Exceptions.exe");

			Console.WriteLine("Assembly: {0}", assembly);
			ShowAssembly(assembly);
		}

		private static void ShowAssembly(Assembly assembly)
		{
			BindingFlags flags = BindingFlags.Default
				| BindingFlags.DeclaredOnly
				| BindingFlags.Instance
				| BindingFlags.Static
				| BindingFlags.Public
				| BindingFlags.NonPublic;

			Type[] types = assembly.GetTypes();
			foreach (Type type in types)
			{
				Console.WriteLine("Type: {0}", type);

				Console.WriteLine("Attributes:");
				ShowAttributes(type);

				Console.WriteLine("Constructors:");
				ConstructorInfo[] constructors = type.GetConstructors(flags);
				foreach (ConstructorInfo constructor in constructors)
				{
					Console.WriteLine("Constructor: {0}", constructor.Name);
				}

				Console.WriteLine("Methods:");
				MethodInfo[] methods = type.GetMethods(flags);
				foreach (MethodInfo method in methods)
				{
					Console.WriteLine("Method: {0}", method.Name);
				}

				if (typeof(IDisposable).IsAssignableFrom(type))
				{
					object instance = Activator.CreateInstance(type);
					MethodInfo methodInfo = type.GetMethod("Dispose");
					methodInfo.Invoke(instance, new object[] { });

					// ако това е типът BookDatabaseV1:
					// object instance = new BookDatabaseV1();
					// instance.Dispose();
				}
			}
		}

		private static void ShowAttributes(MemberInfo memberInfo)
		{
			IEnumerable<Attribute> attributes = memberInfo.GetCustomAttributes<Attribute>(false);
			foreach (Attribute attribute in attributes)
			{
				Console.WriteLine("Attribute: {0}", attribute);
			}
		}
	}
}
