﻿using System;

namespace David.Academy.ProgrammingInCSharp.Delegates
{
	/// <summary>
	/// Делегат за методи с два параметъра от тип double и резултат от тип double
	/// </summary>
	/// <param name="x">Първи параметър</param>
	/// <param name="y">Втори параметър</param>
	/// <returns>Резултат</returns>
	delegate double BinaryMethod(double x, double y);

	/// <summary>
	/// Делегат за методи без параметри и без резултат
	/// (има го вграден в пространството от имена System)
	/// </summary>
	delegate void Action();

	/// <summary>
	/// Шаблонен делегат за методи с един параметър и без резултат
	/// (има го вграден в пространството от имена System)
	/// </summary>
	/// <typeparam name="T">Тип на параметъра</typeparam>
	/// <param name="x">Параметър</param>
	delegate void Action<T>(T x);

	/// <summary>
	/// Шаблонен делегат за методи с два параметъра и без резултат
	/// (има го вграден в пространството от имена System)
	/// </summary>
	/// <typeparam name="T1">Тип на първия параметър</typeparam>
	/// <typeparam name="T2">Тип на втория параметър</typeparam>
	/// <param name="x">Първи параметър</param>
	/// <param name="y">Втори параметър</param>
	delegate void Action<T1, T2>(T1 x, T2 y);

	/// <summary>
	/// Шаблонен делегат за методи без параметри, които връща резултат
	/// (има го вграден в пространството от имена System)
	/// </summary>
	/// <typeparam name="TResult">Тип на резултата</typeparam>
	/// <returns>Резултат</returns>
	delegate TResult Func<TResult>();

	/// <summary>
	/// Шаблонен делегат за методи с един параметър, които връщат резултат
	/// (има го вграден в пространството от имена System)
	/// </summary>
	/// <typeparam name="T">Тип на параметъра</typeparam>
	/// <typeparam name="TResult">Тип на резултата</typeparam>
	/// <param name="x">Параметър</param>
	/// <returns>Резултат</returns>
	delegate TResult Func<T, TResult>(T x);

	/// <summary>
	/// Шаблонен делегат за методи с два параметъра които връщат резултат
	/// </summary>
	/// <typeparam name="T1">Тип на първия параметър</typeparam>
	/// <typeparam name="T2">Тип на втория параметър</typeparam>
	/// <typeparam name="TResult">Тип на резултата</typeparam>
	/// <param name="x">Първи параметър</param>
	/// <param name="y">Втори параметър</param>
	/// <returns>Резултат</returns>
	delegate TResult Func<T1, T2, TResult>(T1 x, T2 y);

	/// <summary>
	/// Делегат за методи с един ref параметър от тип string
	/// </summary>
	/// <param name="str">Параметър, предаван по референция</param>
	delegate void MyDelegate(ref string str);

	// Дефиниционните множества на следните два типа са еквивалентни,
	// но въпреки това те не са съвместими типове данни:
	//BinaryMethod
	//Func<double, double, double>

	/// <summary>
	/// Главен клас на програмата
	/// </summary>
	class Program
	{
		/// <summary>
		/// Умножава две дробни числа
		/// </summary>
		/// <param name="a">Първи множител</param>
		/// <param name="b">Втори множител</param>
		/// <returns>Произведение</returns>
		static double Multiply(double a, double b)
		{
			return a * b;
		}

		/// <summary>
		/// Разделя две дробни числа
		/// </summary>
		/// <param name="a">Делимо</param>
		/// <param name="b">Делител</param>
		/// <returns>Частно</returns>
		static double Divide(double a, double b)
		{
			return a / b;
		}

		/// <summary>
		/// Главна програма
		/// </summary>
		/// <param name="args">Аргументи от командния ред</param>
		static void Main(string[] args)
		{
			// Въвеждане на две дробни числа от конзолата
			Console.Write("A = ");
			double a = double.Parse(Console.ReadLine());
			Console.Write("B = ");
			double b = double.Parse(Console.ReadLine());

			// Деклариране на променлива от делегатен тип
			BinaryMethod method;
			// Избор на операция (умножение или деление);
			// Според избраната операция в променливата method се записва
			// делегатен екземпляр, сочещ към един от методите
			// Multiply(double, double) и Divide(double, double)
			Console.Write("Do you want to multiply (M) or divide (D)?");
			switch (Console.ReadKey(true).Key)
			{
				case ConsoleKey.M:
					// Явно създаване на делегатен екземпляр от тип BinaryMethod,
					// сочещ към метода Multiply(double, double)
					//method = new BinaryMethod(Multiply);
					// Неявно създаване на делегатен екземпляр - еквивалентно на горното
					method = Multiply;
					break;
				case ConsoleKey.D:
					// Явно създаване на делегатен екземпляр от тип BinaryMethod,
					// сочещ към метода Divide(double, double)
					//method = new BinaryMethod(Divide);
					// Неявно създаване на делегатен екземпляр - еквивалентно на горното
					method = Divide;
					break;
				default:
					return;
			}

			// Деклариране на друга променлива от делегатен тип със същото дефиниционно множество от методи
			Func<double, double, double> myFunc;
			// Създаване на делегатен екземпляр от тип Func<double, double, double>,
			// сочещ към метода Multuiply(double, double)
			myFunc = Multiply;
			// Опит за присвояване на съществуващия делегатен екземпляр от тип BinaryMethod
			// на променливата от тип Func<double, double, double>;
			// Предизвиква грешка при компилация, тъй като типовете не са съвместими
			//myFunc = method;
			// Опит за заобикаляне на горната грешка при компилация чрез междинно преобразуване на типове към object;
			// Предизвиква грешка по време на изпълнение, тъй като типовете не са съвместими
			//myFunc = (Func<double, double, double>)(object)method;

			// Изпълняване на метода, към който сочи делегатния екземпляр в променливата method
			double c = method(a, b);
			Console.WriteLine();
			Console.WriteLine("C = {0}", c);

			Console.ReadLine();
		}
	}
}
