﻿namespace David.Academy.ProgrammingInCSharp.Events
{
	/// <summary>
	/// Клас за противник в игра
	/// </summary>
	class Enemy
	{
		/// <summary>
		/// Делегат за обработчици на събитието EnemyDied
		/// </summary>
		/// <param name="enemy">Противник, който е умрял</param>
		public delegate void EnemyDiedHandler(Enemy enemy);

		// Частно поле за обработчиците на ръчно описано събитие
		//private EnemyDiedHandler _enemyDied;

		/// <summary>
		/// Брой точки живот на противника
		/// </summary>
		public int HitPoints { get; private set; }

		/// <summary>
		/// Указва дали протичникът е умрял или не
		/// </summary>
		public bool IsDead { get; private set; }

		// Ръчно описано събитие
		//public event EnemyDiedHandler EnemyDied
		//{
		//    add { _enemyDied += value; }
		//    remove { _enemyDied -= value; }
		//}

		/// <summary>
		/// Събитие, което се предизвиква, когато протиникът умре
		/// </summary>
		public event EnemyDiedHandler EnemyDied;

		/// <summary>
		/// Конструктор на противника с параметър максимален брой точки живот
		/// </summary>
		/// <param name="maxHitPoints">Максимален брой точки живот на противника</param>
		public Enemy(int maxHitPoints)
		{
			HitPoints = maxHitPoints;
		}

		/// <summary>
		/// Прави щети на противника и проверява дали той е умрял
		/// </summary>
		/// <param name="damage">Брой точки щети, които трябва да бъдат направени на противника</param>
		public void Damage(int damage)
		{
			// Ако противникът вече е умрял, не му се правят щети
			if (IsDead)
				return;

			// Щетите се изваждат от точките живот на противника, но точките му живот
			// не могат да паднат под 0
			HitPoints -= damage;
			if (HitPoints < 0)
				HitPoints = 0;

			// Ко точките живот на противника са стигнали 0, то той е умрял
			if (HitPoints == 0)
			{
				IsDead = true;
				// Предизвикване на събитието за смърт на противника
				if (EnemyDied != null)
					EnemyDied(this);
			}
		}
	}
}
