﻿using System.Collections.Generic;
using System.IO;
using System.Xml.Serialization;

namespace David.Academy.ProgrammingInCSharp.FileIO
{
	public class BookDatabase
	{
		public List<Book> Books { get; set; }

		public List<Author> Authors { get; set; }

		public BookDatabase()
		{
			Books = new List<Book>();
			Authors = new List<Author>();
		}

		public void SaveToFile(string fileName)
		{
			using (FileStream stream = File.Open(fileName, FileMode.Create, FileAccess.Write))
			{
				XmlSerializer serializer = new XmlSerializer(typeof(BookDatabase));
				serializer.Serialize(stream, this);
			}
		}

		public static BookDatabase LoadFromFile(string fileName)
		{
			using (FileStream stream = File.Open(fileName, FileMode.Create, FileAccess.Write))
			{
				XmlSerializer serializer = new XmlSerializer(typeof(BookDatabase));
				BookDatabase bookDatabase = (BookDatabase)serializer.Deserialize(stream);

				return bookDatabase;
			}
		}

		public void AddBookFromKbd()
		{
			Book book = Book.CreateFromKbd();
			Books.Add(book);
		}

		public void PrintBooks()
		{
			foreach (Book book in Books)
			{
				book.Print();
			}
		}

		public void AddAuthorFromKbd()
		{
			Author author = Author.CreateFromKbd();
			Authors.Add(author);
		}

		public void PrintAuthors()
		{
			foreach (Author author in Authors)
			{
				author.Print();
			}
		}

		public void SaveBooksToFile(string fileName)
		{
			using (FileStream stream = File.Open(fileName, FileMode.Create, FileAccess.Write))
			using (StreamWriter writer = new StreamWriter(stream))
			{
				foreach (Book book in Books)
				{
					writer.WriteLine(book.Name);
					writer.WriteLine(book.Isbn);
				}
			}
		}

		public void LoadBooksFromFile(string fileName)
		{
			using (FileStream stream = File.Open(fileName, FileMode.Open, FileAccess.Read))
			using (StreamReader reader = new StreamReader(stream))
			{
				Books.Clear();

				while (!reader.EndOfStream)
				{
					Book book = new Book();

					book.Name = reader.ReadLine();
					book.Isbn = reader.ReadLine();

					Books.Add(book);
				}
			}
		}

		public void SaveAuthorsToFile(string fileName)
		{
			using (FileStream stream = File.Open(fileName, FileMode.Create, FileAccess.Write))
			{
				XmlSerializer serializer = new XmlSerializer(Authors.GetType());
				serializer.Serialize(stream, Authors);
			}
		}

		public void LoadAuthorsFromFile(string fileName)
		{
			using (FileStream stream = File.Open(fileName, FileMode.Open, FileAccess.Read))
			{
				XmlSerializer serializer = new XmlSerializer(Authors.GetType());
				Authors = (List<Author>)serializer.Deserialize(stream);
			}
		}
	}
}
